__all__ = ['HTMLAnchorElement',
           'HTMLAppletElement',
           'HTMLBRElement',
           'HTMLBaseElement',
           'HTMLBaseFontElement',
           'HTMLBodyElement',
           'HTMLButtonElement',
           'HTMLCollection',
           'HTMLDListElement',
           'HTMLDirectoryElement',
           'HTMLDivElement',
           'HTMLDocument',
           'HTMLElement',
           'HTMLFieldSetElement',
           'HTMLFontElement',
           'HTMLFormElement',
           'HTMLFrameElement',
           'HTMLFrameSetElement',
           'HTMLHRElement',
           'HTMLHeadElement',
           'HTMLHeadingElement',
           'HTMLHtmlElement',
           'HTMLIFrameElement',
           'HTMLImageElement',
           'HTMLInputElement',
           'HTMLIsIndexElement',
           'HTMLLIElement',
           'HTMLLabelElement',
           'HTMLLegendElement',
           'HTMLLinkElement',
           'HTMLMenuElement',
           'HTMLMetaElement',
           'HTMLModElement',
           'HTMLOListElement',
           'HTMLObjectElement',
           'HTMLOptGroupElement',
           'HTMLOptionElement',
           'HTMLOptionsCollection',
           'HTMLParagraphElement',
           'HTMLParamElement',
           'HTMLPreElement',
           'HTMLQuoteElement',
           'HTMLScriptElement',
           'HTMLSelectElement',
           'HTMLStyleElement',
           'HTMLTableCaptionElement',
           'HTMLTableCellElement',
           'HTMLTableColElement',
           'HTMLTableElement',
           'HTMLTableRowElement',
           'HTMLTableSectionElement',
           'HTMLTextAreaElement',
           'HTMLTitleElement',
           'HTMLUListElement',
           'TAnimateColor']
