/*
 * pyjs runtime.
 *
 * Javascript library required for running code generated by pyjs compiler.
 */
;(function() {
    var py = {
        assert: function() {
            
        },
        floordiv: function(a, b) {
            
        },
        print: function() {
        },
        
        repr: function() {
            
        },
        
        make_args: function(args, star_args, dstar_args) {
            
        }
    };
    
    window.py = py;    
})();
